# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ai/h2o/featurestore/api/v1/FeatureSetProtoApi.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ai/h2o/featurestore/api/v1/FeatureSetProtoApi.proto',
  package='ai.h2o.featurestore.api.v1',
  syntax='proto3',
  serialized_options=b'B\027FeatureSetProtoApiProto',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n3ai/h2o/featurestore/api/v1/FeatureSetProtoApi.proto\x12\x1a\x61i.h2o.featurestore.api.v1\"[\n\x10\x46\x65\x61tureSetHeader\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x02 \x01(\t\x12\x1b\n\x13\x66\x65\x61ture_set_version\x18\x03 \x01(\t*)\n\x0e\x46\x65\x61tureSetType\x12\x07\n\x03RAW\x10\x00\x12\x0e\n\nENGINEERED\x10\x01*m\n\x0b\x46\x65\x61tureType\x12\x17\n\x13\x41UTOMATIC_DISCOVERY\x10\x00\x12\r\n\tCOMPOSITE\x10\x01\x12\r\n\tNUMERICAL\x10\x02\x12\x0f\n\x0b\x43\x41TEGORICAL\x10\x03\x12\x0c\n\x08TEMPORAL\x10\x04\x12\x08\n\x04TEXT\x10\x05\x42\x19\x42\x17\x46\x65\x61tureSetProtoApiProtob\x06proto3'
)

_FEATURESETTYPE = _descriptor.EnumDescriptor(
  name='FeatureSetType',
  full_name='ai.h2o.featurestore.api.v1.FeatureSetType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RAW', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ENGINEERED', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=176,
  serialized_end=217,
)
_sym_db.RegisterEnumDescriptor(_FEATURESETTYPE)

FeatureSetType = enum_type_wrapper.EnumTypeWrapper(_FEATURESETTYPE)
_FEATURETYPE = _descriptor.EnumDescriptor(
  name='FeatureType',
  full_name='ai.h2o.featurestore.api.v1.FeatureType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='AUTOMATIC_DISCOVERY', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='COMPOSITE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NUMERICAL', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CATEGORICAL', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEMPORAL', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TEXT', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=219,
  serialized_end=328,
)
_sym_db.RegisterEnumDescriptor(_FEATURETYPE)

FeatureType = enum_type_wrapper.EnumTypeWrapper(_FEATURETYPE)
RAW = 0
ENGINEERED = 1
AUTOMATIC_DISCOVERY = 0
COMPOSITE = 1
NUMERICAL = 2
CATEGORICAL = 3
TEMPORAL = 4
TEXT = 5



_FEATURESETHEADER = _descriptor.Descriptor(
  name='FeatureSetHeader',
  full_name='ai.h2o.featurestore.api.v1.FeatureSetHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='project_id', full_name='ai.h2o.featurestore.api.v1.FeatureSetHeader.project_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_set_id', full_name='ai.h2o.featurestore.api.v1.FeatureSetHeader.feature_set_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_set_version', full_name='ai.h2o.featurestore.api.v1.FeatureSetHeader.feature_set_version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=83,
  serialized_end=174,
)

DESCRIPTOR.message_types_by_name['FeatureSetHeader'] = _FEATURESETHEADER
DESCRIPTOR.enum_types_by_name['FeatureSetType'] = _FEATURESETTYPE
DESCRIPTOR.enum_types_by_name['FeatureType'] = _FEATURETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FeatureSetHeader = _reflection.GeneratedProtocolMessageType('FeatureSetHeader', (_message.Message,), {
  'DESCRIPTOR' : _FEATURESETHEADER,
  '__module__' : 'ai.h2o.featurestore.api.v1.FeatureSetProtoApi_pb2'
  # @@protoc_insertion_point(class_scope:ai.h2o.featurestore.api.v1.FeatureSetHeader)
  })
_sym_db.RegisterMessage(FeatureSetHeader)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import ai.h2o.featurestore.api.v1.CoreService_pb2 as pb
import ai.h2o.featurestore.api.v1.FeatureSetProtoApi_pb2 as FeatureSetApi

from ..utils import Utils


class MLDatasetFeature:
    def __init__(self, internal_feature: pb.MLDatasetFeature):
        self._internal_feature = internal_feature

    @property
    def name(self):
        return self._internal_feature.name

    @property
    def feature_type(self):
        return FeatureSetApi.FeatureType.Name(self._internal_feature.feature_type)

    @property
    def data_type(self):
        return self._internal_feature.data_type

    @property
    def is_primary_key(self):
        return self._internal_feature.is_primary_key

    def __repr__(self):
        return Utils.pretty_print_proto(self._internal_feature)



import ai.h2o.featurestore.api.v1.CoreService_pb2 as pb
from featurestore.core import interactive_console

from ..retrieve_holder import RetrieveHolder
from ..utils import Utils
from .revert_ingest_job import RevertIngestJob


class Ingest:
    def __init__(self, stub, feature_set, ingest):
        self._stub = stub
        self._feature_set = feature_set
        self._ingest = ingest

    def retrieve(self):
        return RetrieveHolder(self._stub, self._feature_set, "", "", self._ingest.ingest_id)

    @interactive_console.record_stats
    def revert(self):
        job = self.revert_async()
        return job.wait_for_result()

    def revert_async(self) -> RevertIngestJob:
        if self._feature_set.derived_from.HasField("transformation"):
            raise Exception("Manual revert is not allowed on derived feature set")

        request = pb.StartRevertIngestJobRequest()
        request.feature_set.CopyFrom(self._feature_set)
        request.ingest_id = self._ingest.ingest_id
        job_id = self._stub.StartRevertIngestJob(request)
        return RevertIngestJob(self._stub, job_id)

    def __repr__(self):
        return Utils.pretty_print_proto(self._ingest)



import ai.h2o.featurestore.api.v1.CoreService_pb2 as pb

from . import interactive_console
from .config import ConfigUtils


class JobInfo:
    def __init__(self, stub, job_id):
        self._stub = stub
        self._job_id = job_id
        self._next_index = 0

    def show_progress(self):
        if ConfigUtils.is_interactive_print_enabled():
            request = pb.JobProgressInput(job_id=self._job_id.job_id, next_index=self._next_index)
            response = self._stub.GetJobProgress(request)
            if response.progress:
                self._next_index += len(response.progress)
                for progress in response.progress:
                    interactive_console.log(f"Job ID: {self._job_id.job_id}, Status: {progress.message}")

    def get_metrics(self):
        if self._stub.GetJob(self._job_id).done:
            request = pb.JobProgressInput(job_id=self._job_id.job_id, next_index=0)
            response = self._stub.GetJobProgress(request)
            if response.progress:
                return {progress.message: f"{progress.duration_in_seconds}s" for progress in response.progress}
        else:
            interactive_console.log("Job is still running. Please wait until the job finishes.")



import datetime
import time
from typing import Optional

import ai.h2o.featurestore.api.v1.CoreService_pb2 as pb
from ai.h2o.featurestore.api.v1.CoreService_pb2_grpc import CoreServiceStub

from ..collections.ml_datasets import MLDatasets
from ..job_info import JobInfo
from ..utils import Utils
from .query import Query


class FeatureView:
    def __init__(self, stub: CoreServiceStub, feature_view):
        fv = pb.FeatureView()
        fv.CopyFrom(feature_view)
        self._feature_view = fv
        self._stub = stub
        self.ml_datasets = MLDatasets(stub, self)

    @property
    def id(self):
        return self._feature_view.id

    @property
    def version(self):
        return self._feature_view.version

    @property
    def name(self):
        return self._feature_view.name

    @property
    def query(self):
        return Query._from_proto(self._feature_view.query)

    @property
    def description(self):
        return self._feature_view.description

    @description.setter
    def description(self, value):
        update_request = pb.UpdateFeatureViewRequest(
            feature_view_id=self.id,
            feature_view_version=self.version,
            description=value,
        )
        response = self._stub.UpdateFeatureView(update_request)
        fv = pb.FeatureView()
        fv.CopyFrom(response.feature_view)
        self._feature_view = fv

    def __repr__(self):
        return Utils.pretty_print_proto(self._feature_view)

    def create_new_version(self, new_query: Query):
        request = pb.CreateFeatureViewNewVersionRequest(
            feature_view_id=self.id,
            feature_view_version=self.version,
            query=new_query._to_proto(),
        )
        response: pb.FeatureViewResponse = self._stub.CreateFeatureViewNewVersion(request)
        return FeatureView(self._stub, response.feature_view)

    def delete(self):
        request = pb.DeleteFeatureViewRequest(feature_view_id=self.id)
        self._stub.DeleteFeatureView(request)

    def download(
        self,
        start_at: Optional[datetime.datetime] = None,
        end_at: Optional[datetime.datetime] = None,
        output_dir=None,
    ):
        request = pb.RetrieveFeatureViewAsAsLinksRequest(
            feature_view_id=self.id,
            feature_view_version=self.version,
            start_at=Utils.date_time_to_proto_timestamp(start_at),
            end_at=Utils.date_time_to_proto_timestamp(end_at),
        )
        job_id = self._stub.StartRetrieveFeatureViewAsLinksJob(request)

        info = JobInfo(self._stub, job_id)
        while not self._stub.GetJob(job_id).done:
            info.show_progress()
            time.sleep(2)
        info.show_progress()  # there is possibility that some progress was pushed before finishing job
        response = self._stub.GetRetrieveFeatureViewAsLinksJobOutput(job_id)

        return Utils.download_files(output_dir, response.download_links)

    def as_spark_frame(
        self,
        spark_session,
        start_at: Optional[datetime.datetime] = None,
        end_at: Optional[datetime.datetime] = None,
    ):
        from ..commons.spark_utils import SparkUtils

        session_id = spark_session.conf.get("ai.h2o.featurestore.sessionId", "")
        request = pb.RetrieveFeatureViewAsSparkRequest(
            feature_view_id=self.id,
            feature_view_version=self.version,
            start_at=Utils.date_time_to_proto_timestamp(start_at),
            end_at=Utils.date_time_to_proto_timestamp(end_at),
            session_id=session_id,
        )
        job_id = self._stub.StartRetrieveFeatureViewAsSparkJob(request)

        info = JobInfo(self._stub, job_id)
        while not self._stub.GetJob(job_id).done:
            info.show_progress()
            time.sleep(2)
        info.show_progress()  # there is possibility that some progress was pushed before finishing job
        response = self._stub.GetRetrieveFeatureViewAsSparkJobOutput(job_id)

        spark_session.conf.set("ai.h2o.featurestore.sessionId", response.session_id)

        SparkUtils.configure_user_spark(spark_session)
        for k, v in response.options.items():
            spark_session.conf.set(k, v)

        return spark_session.read.format("parquet").load(response.path)



from __future__ import annotations

from .feature_set_ref import FeatureSetRef


class FeatureRef:
    def __init__(self, name, feature_set_ref: FeatureSetRef):
        self.name = name
        self.feature_set_ref = feature_set_ref

    def __repr__(self):
        return f"FeatureRef({self.name}, {self.feature_set_ref})"

    def __hash__(self):
        return hash((self.name, self.feature_set_ref))

    def __eq__(self, other):
        return self.name == other.name and self.feature_set_ref == other.feature_set_ref
